// SNF4CGP/JobPool.cpp
// Copyright (C) 2009 ARM Research Labs, LLC.
// See www.armresearch.com for more information.

#include "JobPool.hpp"
#include "Command.hpp"
#include "ScannerPool.hpp"

#include "../SNFMulti/SNFMulti.hpp"

#include "../CodeDweller/threading.hpp"
#include "../CodeDweller/faults.hpp"

#include <iostream>
#include <vector>

using namespace std;

//// Tuning Constants //////////////////////////////////////////////////////////

const int ReadBufferSize = snf_ScanHorizon;
const int StringReserveSize = 2048;

//// Job ///////////////////////////////////////////////////////////////////////

void Job::emitCommentPrefix() {
}

void Job::emitComment(string Comment) {
}

void Job::emitResponsePrevix() {
}

void Job::emitOK() {
}

void Job::emitINTF() {
}

void Job::emitFAIL() {
}

void Job::emitADDHEADER(string Headers) {
}

void Job::emitERROR() {
}

void Job::emitDISCARD() {
}

void job::finalize() {
}

void Job::doWakeUp() {
}

void Job::doINTF() {
}

void Job::doFAIL() {
}

void Job::doFILE() {
}

void Job::readTopOfMessage(ifstream& Reader) {
}

void Job::doRead() {
}

void Job::doScan() {
}

void Job::doAction() {
}

void Job::doBypass() {
}

void Job::doAllow() {
}

void Job::doReject() {
}

void Job::doDelete() {
}

void Job::moveMessageToHoldPath(ifstream& Reader) {
}

void Job::doHold() {
}

Job::Job(ScannerPool& S, OutputProcessor& O) :                                  // Construct with important links.
  Scanners(S),
  Output(O),
  ScanResultCode(0),
  ReadLength(0) {                                                               // Minimize heap thrashing.
      OutputBuffer.reserve(StringReserveSize);
      HeadersToInject.researve(StringReserveSize);
      ReadBuffer.reserve(ReadBufferSize);
}

Job::~Job() {                                                                   // Cleanup when destructing.
    try { clear(); }
    catch(...) {}
}

void Job::clear() {                                                             // Cleanup for the next command.
    CurrentCommand.clear();
    ScanResultCode = 0;
    OutputBuffer.clear();
    HeadersToInject.clear();
    MessageMoveFilePath.clear();
    ReadBuffer.clear();
    ReadLength = 0;
}

void Job::setCommand(Command& C) {                                              // Assign a command for this job.
    CurrentCommand = C;
}

void Job::doIt() {                                                              // Get the job done.
    switch(CurrentCommand.Type) {
        case Command::WAKE: { doWakeUp(); break; }
        case Command::INTF: { doINTF(); break; }
        case Command::FILE: { doFILE(); break; }
        default: { doFAIL(); break; }
    }
    finalize();
}

string& Job::Responses() {                                                      // Read the job's report.
    return OutputBuffer;
}

//// JobPool ///////////////////////////////////////////////////////////////////

JobPool::JobPool() :                                                            // Simple constructor.
  Output_(0),
  Scanners_(0),
  AllocatedJobs(0),
  Started(false) {
}

JobPool::~JobPool() {                                                           // Clean up on the way out.
    try { stop(); }
    catch(...) {}
}

// Watch out -- Initializing the JobPool also means starting up SNFMulti and
// the ScannerPool.

void JobPool::init(string Configuration, OutputProcessor& O) {                  // Initialize the JobPool.
}

Job* JobPool::makeJob() {                                                       // Create and count a Job.
}

Job& JobPool::grab() {                                                          // Grab a job (prefer from the pool).
}

void JobPool::drop(Job& J) {                                                    // Drop a job into the pool.
}

void JobPool::killJobFromPool() {                                               // Kill and count a Job from the pool.
}

// Watch out -- Stopping the JobPool also means shutting down SNFMulti and
// the ScannerPool.

void JobPool::stop() {                                                          // Shut down the JobPool.
    // Kill off all of the allocated Jobs.
    // Shut down and destroy the ScannerPool.
}

